/*
   This example is part of the Javadoc for the Condition class.

   https://docs.oracle.com/en/java/javase/21/docs/api/java.base/java/util/concurrent/locks/Condition.html
*/

import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;

public class BoundedBufferGeneric<E>
{
   final Lock lock = new ReentrantLock();
   final Condition notFull  = lock.newCondition();
   final Condition notEmpty = lock.newCondition();

   final Object[] items = new Object[100];
   int putptr, takeptr, count;

   public void put(E x) throws InterruptedException
   {
      lock.lock();
      try
      {
         while (count == items.length)
            notFull.await();

         items[putptr] = x;
         if (++putptr == items.length) putptr = 0;
         ++count;

         notEmpty.signal();
      }
      finally
      {
         lock.unlock();
      }
   }

   public E take() throws InterruptedException
   {
      lock.lock();
      try
      {
         while (count == 0)
            notEmpty.await();

         @SuppressWarnings("unchecked")
         E x = (E) items[takeptr];
         if (++takeptr == items.length) takeptr = 0;
         --count;

         notFull.signal();
         return x;
      }
      finally
      {
         lock.unlock();
      }
   }
}
