/*
   Adapted from
   Chapter 3, Waiting and Notification,
   Listing 3-2, pages 46-47,
   "Java Threads and the Concurrency Utilities"
   by Jeff Friesen, Apress, 2015.
*/

import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.Condition;

public class PC3
{
   public static final ReentrantLock lock = new ReentrantLock();
   public static final Condition writeableC = lock.newCondition();

   public static void main(String[] args)
   {
      Shared s = new Shared();
      new Producer(s).start();
      new Consumer(s).start();
   }


   static class Shared
   {
      private char c;
      private volatile boolean writeable = true;

      void setSharedChar(char c)
      {
         PC3.lock.lock(); // This is why the lock is "reentrant".
         while (!writeable)
            try
            {
               PC3.writeableC.await();
            }
            catch (InterruptedException ie)
            {
            }
         this.c = c;
         writeable = false;
         PC3.writeableC.signal();
         PC3.lock.unlock();
      }


      char getSharedChar()
      {
         PC3.lock.lock();
         while (writeable)
            try
            {
               PC3.writeableC.await();
            }
            catch (InterruptedException ie)
            {
            }
         writeable = true;
         PC3.writeableC.signal();
         PC3.lock.unlock();
         return c;
      }
   }


   static class Producer extends Thread
   {
      private final Shared s;

      Producer(Shared s)
      {
         this.s = s;
      }

      @Override
      public void run()
      {
         for (char ch = 'A'; ch <= 'Z'; ch++)
         {
            PC3.lock.lock(); // This is why the lock is "reentrant".
            {
               s.setSharedChar(ch);  // setSharedChar() will relock the lock
               System.out.println(ch + " produced by producer.");
            }
            PC3.lock.unlock();
         }
      }
   }


   static class Consumer extends Thread
   {
      private final Shared s;

      Consumer(Shared s)
      {
         this.s = s;
      }

      @Override
      public void run()
      {
         char ch;

         do
         {
            try{Thread.sleep(1000);}catch(Exception e){} // Slow down the consumer.

            PC3.lock.lock();
            {
               ch = s.getSharedChar();
               System.out.println(ch + " consumed by consumer.");
            }
            PC3.lock.unlock();
         }
         while (ch != 'Z');
      }
   }
}
