/*
   Adapted from
   Chapter 3, Waiting and Notification,
   Listing 3-2, pages 46-47,
   "Java Threads and the Concurrency Utilities"
   by Jeff Friesen, Apress, 2015.

   This is a broken version of PC2 and PC3. It adds
   a mutex lock instead of the nested locking on a
   single lock. This shows how easy it is to
   misunderstand the nature of locking.
*/

import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.Condition;

public class PC4
{
   public static final ReentrantLock lock = new ReentrantLock();
   public static final ReentrantLock mutex = new ReentrantLock(); // Added a mutex lock.
   public static final Condition writeableC = lock.newCondition();

   public static void main(String[] args)
   {
      Shared s = new Shared();
      new Producer(s).start();
      new Consumer(s).start();
   }


   static class Shared
   {
      private char c;
      private volatile boolean writeable = true;

      void setSharedChar(char c)
      {
         PC4.lock.lock();
         while (!writeable)
            try
            {
               PC4.writeableC.await();
            }
            catch (InterruptedException ie)
            {
            }
         this.c = c;
         writeable = false;
         PC4.writeableC.signal();
         PC4.lock.unlock();
      }


      char getSharedChar()
      {
         PC4.lock.lock();
         while (writeable)
            try
            {
               PC4.writeableC.await();
            }
            catch (InterruptedException ie)
            {
            }
         writeable = true;
         PC4.writeableC.signal();
         PC4.lock.unlock();
         return c;
      }
   }


   static class Producer extends Thread
   {
      private final Shared s;

      Producer(Shared s)
      {
         this.s = s;
      }

      @Override
      public void run()
      {
         for (char ch = 'A'; ch <= 'Z'; ch++)
         {
            PC4.mutex.lock();  // This causes deadlock!
            {
               s.setSharedChar(ch);  // Nested synchronization.
               System.out.println(ch + " produced by producer.");
            }
            PC4.mutex.unlock();
         }
      }
   }


   static class Consumer extends Thread
   {
      private final Shared s;

      Consumer(Shared s)
      {
         this.s = s;
      }

      @Override
      public void run()
      {
         char ch;

         do
         {
            //try{Thread.sleep(1000);}catch(Exception e){} // Slow down the consumer.

            PC4.mutex.lock();  // This causes deadlock!
            {
               ch = s.getSharedChar(); // Nested synchronization.
               System.out.println(ch + " consumed by consumer.");
            }
            PC4.mutex.unlock();
         }
         while (ch != 'Z');
      }
   }
}
