/**
   This program "consumes" integer values from stdin.
   The rate of consumption is set by a command-line argument.

   Pipe the output from Pipeline_Producer.java
   into the input of this program.

   Make the producer faster than the consumer, so the
   producer will keep the buffer between them full.
   > java Pipeline_Producer 10 | java Pipeline_Consumer 100
   > java Pipeline_Producer 1  | java Pipeline_Consumer 10

   Make the consumer faster than the producer and give
   the producer a head start. The consumer will drain the buffer.
   > java Pipeline_Producer 500 50 | java Pipeline_Consumer 200 500
*/
import java.util.Scanner;
public class Pipeline_Consumer
{
   public static void main(String[] args)
   {
      final Scanner in = new Scanner(System.in);
      int milliSeconds = 500;
      int headStart = 0;  // Give the producer a head start (in milliseconds).

      if (args.length > 0)
      {
         milliSeconds = Integer.parseInt(args[0]);
      }
      if (args.length > 1)
      {
         headStart = Integer.parseInt(args[1]);
      }

      // Give the producer a head start.
      try
      {
         Thread.sleep(headStart);
      }
      catch (InterruptedException e)
      {
         e.printStackTrace();
      }

      while(in.hasNextInt())
      {
         final int counter = in.nextInt();
         System.out.println("\t"+counter);
         try // Slow down the consumer.
         {
            Thread.sleep(milliSeconds);
         }
         catch (InterruptedException e)
         {
            e.printStackTrace();
         }
      }
   }
}
