/**
   This program "produces" integer values to stdout.
   The rate of production is set by a command-line argument.

   Pipe the output from this program
   into the input of Pipeline_Consumer.java.

   Make the producer faster than the consumer, so the
   producer will keep the buffer between them full.
   > java Pipeline_Producer 10 | java Pipeline_Consumer 100
   > java Pipeline_Producer 1  | java Pipeline_Consumer 10

   Make the consumer faster than the producer and give
   the producer a head start. The consumer will drain the buffer.
   > java Pipeline_Producer 500 50 | java Pipeline_Consumer 200 500
*/

public class Pipeline_Producer
{
   public static void main(String[] args) throws java.io.IOException
   {
      final int LIMIT = 10_000;  // or Integer.MAX_VALUE;
      int milliSeconds = 500;
      int headStart = 0;  // Give the producer a head start.

      if (args.length > 0)
      {
         milliSeconds = Integer.parseInt(args[0]);
      }
      if (args.length > 1)
      {
         headStart = Integer.parseInt(args[1]);
      }

      // Give the producer a head start.
      int counter = 0;
      for (int i = 0; i < headStart; ++i)
      {
            System.err.print(counter + "\n");
            System.out.print(counter + "  ");
            if (System.out.checkError()) throw new java.io.IOException();
            ++counter;
      }

      while(counter <= LIMIT)
      {
         System.err.print(counter + "\n");
         System.out.print(counter + "  ");
         if (System.out.checkError()) throw new java.io.IOException();
         ++counter;
         try // Slow down the producer.
         {
            Thread.sleep(milliSeconds);
         }
         catch (InterruptedException e)
         {
            e.printStackTrace();
         }
      }
   }
}

// https://docs.oracle.com/en/java/javase/22/docs/api/java.base/java/io/PrintStream.html
// https://docs.oracle.com/en/java/javase/22/docs/api/java.base/java/io/PrintStream.html#checkError()
